@@ -66,7 +66,7 @@ BEGIN
   if ($error) {warn "$error -> gstreamer output won't be available.\n"}
   if ($GST_ok)
   {	$::Options{gst_sink}||= (grep ($reg->lookup_feature($_.'sink'), qw/autoaudio gconfaudio alsa esd pulse oss/),'alsa')[0]; #find a default sink
-	if ($reg->lookup_feature('equalizer-10bands')) { $GST_EQ_ok=1; }
+	if (my $feat=$reg->lookup_feature('equalizer-10bands')) { $GST_EQ_ok=1; }
 	else {warn "gstreamer plugin 'equalizer-10bands' not found -> equalizer not available\n";}
 	if ($reg->lookup_feature('rglimiter') && $reg->lookup_feature('rgvolume')) { $GST_RG_ok=1; }
 	else {warn "gstreamer plugins 'rglimiter' and/or 'rgvolume' not found -> replaygain not available\n";}
@@ -265,6 +265,32 @@ sub set_equalizer
 	$::Options{gst_equalizer}=join ':',@vals;
 	::setlocale(::LC_NUMERIC, '');
 }
+sub EQ_Get_Range
+{	my ($min,$max)=(-1,1);
+	{	my $equalizer=$PlayBin->get_by_name('equalizer')
+		 || GStreamer::ElementFactory->make('equalizer-10bands' => 'equalizer');
+		last unless $equalizer;
+		my $prop= $equalizer->find_property('band0');
+		last unless $prop;
+		$min=$prop->get_minimum;
+		$max=$prop->get_maximum;
+	}
+	my $unit= ($max==1 && $min==-1) ? '' : 'dB';
+	return ($min,$max,$unit);
+}
+sub EQ_Get_Hz
+{	my $i=$_[1];
+	my $equalizer=$PlayBin->get_by_name('equalizer')
+	 || GStreamer::ElementFactory->make('equalizer-10bands' => 'equalizer');
+	return undef unless $equalizer;
+	my $hz= $equalizer->find_property('band'.$i)->get_nick;
+	if ($hz=~m/^(\d+)\s*(k?)Hz/)
+	{	$hz=$1; $hz*=1000 if $2;
+		$hz= $hz>=1000 ? sprintf '%.1fkHz',$hz/1000 :
+				 sprintf '%dHz',$hz ;
+	}
+	return $hz;
+}
 
 sub create_visuals
 {	unless ($VSink)
