@@ -204,6 +204,7 @@ my %objects=
 		click3	=> sub {::PopupContextMenu(\@::SongCMenu,{mode=> 'P', self=> $_[0], IDs => [$::SongID]}) if defined $::SongID;},
 		dragsrc => [::DRAG_ID,sub {$::SongID}],
 		dragdest=> [::DRAG_ID,sub {::Select(song => $_[2]);}],
+		cursor	=> 'hand2',
 	},
 	Title_by =>
 	{	class	=> 'Layout::Label',
@@ -213,6 +214,7 @@ my %objects=
 		click1	=> \&::ChooseSongsFromA_current,
 		click3	=> sub {::PopupContextMenu(\@::SongCMenu,{mode=> 'P', self=> $_[0], IDs => [$::SongID]}) if defined $::SongID;},
 		dragsrc => [::DRAG_ID,sub {$::SongID}],
+		cursor	=> 'hand2',
 	},
 	Artist =>
 	{	class	=> 'Layout::Label',
@@ -221,6 +223,7 @@ my %objects=
 		click1	=> sub { ::PopupAA(::SONG_ARTIST) if defined $::SongID; },
 		click3	=> sub { ::ArtistContextMenu($::Songs[$::SongID][::SONG_ARTIST],{self =>$_[0], ID=>$::SongID, mode => 'P'}) if defined $::SongID; },
 		dragsrc => [::DRAG_ARTIST,sub {$::Songs[$::SongID][::SONG_ARTIST]}],
+		cursor	=> 'hand2',
 	},
 	Album =>
 	{	class	=> 'Layout::Label',
@@ -229,6 +232,7 @@ my %objects=
 		click1	=> sub { ::PopupAA(::SONG_ALBUM,$::Songs[$::SongID][::SONG_ARTIST]) if defined $::SongID; },
 		click3	=> sub { ::PopupContextMenu(\@::cMenuAA,{self =>$_[0], col=>::SONG_ALBUM, ID=>$::SongID, key=>$::Songs[$::SongID][::SONG_ALBUM], mode => 'P'}) if defined $::SongID; },
 		dragsrc => [::DRAG_ALBUM,sub {$::Songs[$::SongID][::SONG_ALBUM]}],
+		cursor	=> 'hand2',
 	},
 	Date =>
 	{	class	=> 'Layout::Label',
@@ -434,6 +438,7 @@ my %objects=
 	},
 	SimpleSearch =>
 	{	New	=> sub { SimpleSearch->new($_[0],$_[1]); },
+		dragdest => [::DRAG_FILTER,sub { ::SetFilter($_[0],$_[2]);}],
 	},
 	LabelsIcons =>
 	{	New	=> sub { Gtk2::Table->new(1,1); },
@@ -491,30 +496,9 @@ my %objects=
 		update	=> \&PlayOrderComboUpdate,
 	},
 	Equalizer =>
-	{	New => sub
-		{	my $opt1=$_[0];
-			my $box=Gtk2::HBox->new;
-			for my $i (0..9)
-			{	my $adj=Gtk2::Adjustment->new(0, -1, 1, .05, .1,0);
-				my $scale=Gtk2::VScale->new($adj);
-				$scale->set_inverted(1);
-				$box->{'adj'.$i}=$adj;
-				$adj->signal_connect(value_changed =>
-					sub { $::Play_package->set_equalizer($_[1],$_[0]->get_value) unless $_[0]{busy}; ::HasChanged('Equalizer') },$i);
-				if ($opt1->{labels} || 1)
-				{	my $vbox=Gtk2::VBox->new;
-					$vbox->add($scale);
-					my $label=Gtk2::Label->new;
-					$label->set_markup('<small>'. (qw/20 43 93 200 430 928 2k 4.3k 9.3k 20k/)[$i] . 'Hz</small>');
-					$vbox->pack_start($label,0,0,0);
-					$scale=$vbox;
-				}
-				$box->pack_start($scale,0,0,2);
-			}
-			return $box;
-		},
+	{	New => \&Layout::Equalizer::new,
 		event => 'Equalizer',
-		update => sub { my $box=$_[0]; my $ok= $::Play_package->{EQ} && $::Options{gst_use_equalizer}; $box->set_sensitive($ok); return unless $ok; my @val= split /:/, $::Options{gst_equalizer}; for my $i (0..9) {my $adj=$box->{'adj'.$i}; $adj->{busy}=1; $adj->set_value($val[$i]); delete $adj->{busy}; } },
+		update => \&Layout::Equalizer::update,
 	},
 	RadioList =>
 	{	New => sub {GMB::RadioList->new},
@@ -719,7 +703,7 @@ sub Pack
 		if ( $names[0]=~m/^\d+$/ )
 		{	my $s=shift @names;
 			my @req=($mode eq 'vertical')? (-1,$s) : ($s,-1);
-			$widgets->{$names[0]}->set_size_request(@req);
+			$widgets->{$_}->set_size_request(@req) for @names;
 			next if @names==1;
 		}
 		my $sizegroup=Gtk2::SizeGroup->new($mode);
@@ -859,6 +843,9 @@ sub NewObject
 	{	$widget->{actions}{activate}=$activate;
 		$widget->signal_connect(clicked => \&Button_activate_cb);
 	}
+	if (my $cursor= ($opt1->{cursor}||$ref->{cursor}))
+	{	$widget->signal_connect(realize => sub { $_[0]->window->set_cursor(Gtk2::Gdk::Cursor->new($_[1])); },$cursor);
+	}
 
 	my $tip= $ref->{tip};
 	$tip= $opt1->{tip} if exists $opt1->{tip} && (!$tip || !ref $tip);
@@ -984,6 +971,7 @@ sub Hide
 		$widget->hide;
 	}
 	$self->resize($ww,$wh) if $resize && $resize ne '_';
+	::HasChanged('HiddenWidgets');
 }
 sub Show
 {	my ($self,$names,$resize)=@_;
@@ -1004,6 +992,18 @@ sub Show
 		}
 	}
 	$self->resize($ww,$wh) if $resize && $resize ne '_';
+	::HasChanged('HiddenWidgets');
+}
+
+sub GetShowHideState
+{	my ($self,$names)=@_;
+	my $hidden;
+	for my $name ( split /\|/,$names )
+	{	my $widget=$self->{widgets}{$name};
+		next unless $widget;
+		$hidden++ unless $widget->visible;
+	}
+	return !$hidden;
 }
 
 sub ToggleFullscreen
@@ -1055,11 +1055,23 @@ sub SetFocusOn
 {	my ($self,$name)=@_;
 	my $widget=$self->{widgets}{$name};
 	if ($widget)
-	{	$widget=$widget->{DefaultFocus}||$widget;
+	{	$widget=$widget->{DefaultFocus} while $widget->{DefaultFocus};
+		TurnPagesToWidget($widget);
 		$widget->grab_focus;
 	}
 }
 
+sub TurnPagesToWidget #change the current page of all parent notebook so that widget is on it
+{	my $parent=$_[0];
+	while (1)
+	{	my $child=$parent;
+		$parent=$child->parent;
+		last unless $parent;
+		if ($parent->isa('Gtk2::Notebook'))
+		 { $parent->set_current_page($parent->page_num($child)); }
+	}
+}
+
 #################################################################################
 
 sub RemoveFilter
@@ -1362,6 +1374,7 @@ sub new
 	#	$songlist->FollowSong;
 	#}
 	$self->init;
+	::HasChanged('HiddenWidgets');
 	return $self;
 }
 sub init
@@ -1435,7 +1448,7 @@ sub SetWindowOptions
 		$self->set_keep_above(1) if $wstate{ontop};
 		$self->set_keep_below(1) if $wstate{below};
 		$self->set_decorated(0)  if $wstate{nodecoration};
-		$self->{hidden}={ split ':',$wstate{hidden} } if $wstate{hidden};
+		$self->{hidden}={ $wstate{hidden}=~m/(\w+)(?::?(\d+x\d+))?/g } if $wstate{hidden};
 	}
 	else	#old format (v<0.9568)
 	{	my @wopt=split /,/,$wopt;
@@ -1454,6 +1467,7 @@ sub SetWindowOptions
 	  $self->{x}=$x; $self->{y}=$y;	#move before show_all rarely works, so save pos to set it later
 	}
 	my $title=$Layouts{$layout}{Title} || _"%S by %a";
+	$title=~s/^"(.*)"$/$1/;
 	if (my @l=::UsedFields($title)) { $self->{TitleString}=$title; $self->{towatch}{$_}=undef for @l; }
 	else { $self->set_title($title) }
 }
@@ -1623,11 +1637,12 @@ sub SimpleAdd
 }
 
 sub NewTB
-{	my $opt=$_[1];
+{	my ($opt1,$opt2)=@_;
 	my $nb=Gtk2::Notebook->new;
 	$nb->set_scrollable(::TRUE);
 	$nb->popup_enable;
-	if ($opt && $opt=~m/page=(\d+)/) { $nb->{SetPage}=$1; }
+	if ($opt1->{tabpos}) {$nb->set_tab_pos($opt1->{tabpos});}
+	if ($opt2 && $opt2=~m/page=(\d+)/) { $nb->{SetPage}=$1; }
 	$nb->{SaveOptions}=sub { 'page='.$_[0]->get_current_page };
 	return $nb;
 }
@@ -1636,8 +1651,8 @@ sub PackTB
 	$title=~s/^"// && $title=~s/"$//;
 	$nb->append_page($wg, Gtk2::Label->new($title) );
 	$nb->set_tab_reorderable($wg,::TRUE);
-	my $n=$nb->{SetPage};
-	if (defined $n && $n==($nb->get_n_pages-1)) {$wg->show; $nb->set_current_page($n);}
+	my $n=$nb->{SetPage}||0;
+	if ($n==($nb->get_n_pages-1)) {$wg->show; $nb->set_current_page($n); $nb->{DefaultFocus}=$wg; }
 }
 
 sub SBoxPack
@@ -1787,8 +1802,8 @@ package Layout::Label;
 use Gtk2;
 
 use constant
-{	SPEED	=> 100,	#timeout in ms
-	INCR	=> 5,	#scroll increment in pixel
+{	SPEED	=> 20,	#timeout in ms
+	INCR	=> 1,	#scroll increment in pixel
 };
 
 use base 'Gtk2::EventBox';
@@ -1877,8 +1892,6 @@ sub expose_cb #only for scrollable labels
 	$xoffset+=($aw-$lw)*$xalign if $aw>$lw;
 	$yoffset+=($ah-$lh)*$yalign if $ah>$lh;
 	my $gc=$label->get_style->text_gc($label->state);
-	#$gc->set_clip_rectangle(undef); #sometimes needed when scrolling, don't know why
-	$gc->set_clip_rectangle(Gtk2::Gdk::Rectangle->new(0,0,$lw+$xoffset,$lh+$yoffset)); #undef is not accepted with some old binding version ($Gtk2::VERSION <1.120 ?)
 	my $pixmap=Gtk2::Gdk::Pixmap->new($label->window, $lw,$lh, -1);
 	$pixmap->draw_drawable($gc, $label->window, $xoffset,$yoffset, $label->{dx},0,$aw,$ah);
 	$pixmap->draw_layout($gc, 0,0, $layout);
@@ -2075,7 +2088,7 @@ sub set
 		delete $self->{size} unless $self->{pixbuf} || $::ToDo{'8_LoadImg'.$self};
 		$self->set_size_request(-1,-1) unless $self->{forceratio};
 		$self->queue_resize;
-       	}) if $self->get_toplevel->{natural_size};
+	}) if $self->get_toplevel->{natural_size};
 }
 
 sub size_allocate_cb
@@ -2143,20 +2156,28 @@ sub new
 	#$self->{gravity}=$opt1{gravity};
 	$self->{widget}=$opt1->{widget};
 	$self->{resize}=$opt1->{resize};
-	$self->{SaveOptions}=\&SaveOptions;
-	$self->set_active(1) if $opt2->{state};
 	$self->signal_connect( toggled => \&toggled_cb );
+	::Watch($self,'HiddenWidgets',\&UpdateToggleState);
 
 	return $self;
 }
 
-sub SaveOptions
-{	'state='. ( $_[0]->get_active ? '1' : '0' );
+sub UpdateToggleState
+{	my $self=$_[0];
+	return unless $self->{widget};
+	my $layw=::get_layout_widget($self);
+	return unless $layw;
+	my $state=$layw->GetShowHideState($self->{widget});
+	$self->{busy}=1;
+	$self->set_active($state);
+	delete $self->{busy};
 }
 
 sub toggled_cb
 {	my $self=$_[0];
+	return if $self->{busy} || !$self->{widget};
 	my $layw=::get_layout_widget($self);
+	return unless $layw;
 	if ($self->get_active)	{ $layw->Show($self->{widget},$self->{resize}) }
 	else			{ $layw->Hide($self->{widget},$self->{resize}) }
 }
@@ -2176,9 +2197,8 @@ sub new
 	if ($opt1->{togglewidget})
 	{	$self->{widget}=$opt1->{togglewidget};
 		$self->{resize}=$opt1->{resize};
-		$self->{SaveOptions}=sub {{ state=> $_[0]->get_active }};
-		$self->set_active(1) if $opt2->{state};
-		$self->signal_connect( toggled => \&toggled_cb );
+		$self->signal_connect( toggled => \&Layout::TogButton::toggled_cb );
+		::Watch($self,'HiddenWidgets',\&Layout::TogButton::UpdateToggleState);
 	}
 	if ($opt1->{command})
 	{	$self->signal_connect(activate => \&::run_command,$opt1->{command});
@@ -2187,22 +2207,6 @@ sub new
 	return $self;
 }
 
-sub get_player_window
-{	my $menu=$_[0]->parent;
-	while (ref $menu eq 'Gtk2::Menu')
-	{	$menu=$menu->get_attach_widget->parent;
-	}
-	return ::get_layout_widget($menu);
-}
-
-sub toggled_cb
-{	my $self=$_[0];
-	my $layw=get_player_window($self);
-	return unless $layw;
-	if ($self->get_active)	{ $layw->Show($self->{widget},$self->{resize}) }
-	else			{ $layw->Hide($self->{widget},$self->{resize}) }
-}
-
 package GMB::Context;
 use Gtk2;
 our %Contexts;
@@ -2379,7 +2383,73 @@ sub Changed
 	}
 }
 
+package Layout::Equalizer;
+sub new
+{	my $opt1=$_[0];
+	my $self=Gtk2::HBox->new(1,0); #homogenous
+	for my $i (0..9)
+	{	my $adj=Gtk2::Adjustment->new(0, -1, 1, .05, .1,0);
+		my $scale=Gtk2::VScale->new($adj);
+		$scale->set_draw_value(0);
+		$scale->set_inverted(1);
+		$self->{'adj'.$i}=$adj;
+		$adj->signal_connect(value_changed =>
+		sub { $::Play_package->set_equalizer($_[1],$_[0]->get_value) unless $_[0]{busy}; ::HasChanged('Equalizer','value') },$i);
+		$self->{labels}= $opt1->{labels} || 'x-small';
+		$self->{labels}=undef if $self->{labels} eq 'none';
+		if ($self->{labels})
+		{	my $vbox=Gtk2::VBox->new;
+			my $label0=Gtk2::Label->new;
+			$vbox->pack_start($label0,0,0,0);
+			$self->{'Valuelabel'.$i}=$label0;
+			$vbox->add($scale);
+			my $label1=Gtk2::Label->new;
+			$vbox->pack_start($label1,0,0,0);
+			$self->{'Hzlabel'.$i}=$label1;
+			$scale=$vbox;
+		}
+		$self->pack_start($scale,1,1,2);
+	}
+	return $self;
+}
 
+sub update
+{	my ($self,$event)=@_;
+	my $ok= $::Play_package->{EQ} && $::Options{gst_use_equalizer};
+	$self->set_sensitive($ok);
+	if ((!$event || $event eq 'package') && $self->{labels})
+	{	my ($min,$max,$unit)= $::Play_package->{EQ} ? $::Play_package->EQ_Get_Range : (-1,1,'');
+		my $inc=abs($max-$min)/10;
+		$unit=' '.$unit if $unit;
+		$self->{unit}= $unit;
+		for my $i (0..9)
+		{	if ($self->{labels})
+			{	my $val='-';
+				$val=::PangoEsc($::Play_package->EQ_Get_Hz($i)||'?') if $::Play_package->{EQ};
+				$self->{'Hzlabel'.$i}->set_markup(qq(<span size="$self->{labels}">$val</span>));
+			}
+			my $adj=$self->{'adj'.$i};
+			$adj->{busy}=1;
+			$adj->lower($min);
+			$adj->upper($max);
+			$adj->step_increment($inc/10);
+			$adj->page_increment($inc);
+			delete $adj->{busy};
+		}
+	}
+	my @val= split /:/, $::Options{gst_equalizer};
+	#@val=(0)x10 unless $ok;
+	for my $i (0..9)
+	{	my $val=$val[$i];
+		my $adj=$self->{'adj'.$i};
+		$adj->{busy}=1;
+		$adj->set_value($val);
+		delete $adj->{busy};
+		next unless $self->{labels};
+		$val=sprintf('%.1f',$val).$self->{unit};
+		$self->{'Valuelabel'.$i}->set_markup(qq(<span size="$self->{labels}">$val</span>)) if $self->{labels};
+	}
+}
 
 package SVBox;
 use Gtk2;
@@ -2532,7 +2602,7 @@ sub draw
 	$x+=int ($w-$pbw)/2;
 	$y+=int ($h-$pbh)/2;
 	my $style=$widget->get_style;
-	my $gc=$style->fg_gc($state1);
+	my $gc=Gtk2::Gdk::GC->new($widget->window);
 	$gc->set_clip_rectangle($event->area);
 	$widget->window->draw_pixbuf($gc,$pb,0,0,$x,$y,$pbw,$pbh,'none',0,0);
 	$style->paint_focus($widget->window, $state1, $event->area, $widget, undef, $x,$y,$pbw,$pbh) if $widget->has_focus;
