@@ -94,7 +94,7 @@ sub Read
 		push @id3v2s,$filetag->{ID3v2} if $filetag->{ID3v2};
 		push @id3v2s,@{ $filetag->{ID3v2s} } if $filetag->{ID3v2s};
 		for my $ID3v2 ( @id3v2s )
-		{	for my $fname (qw/TIT2 TPE1 TALB TYER TRCK TPOS TIT3 COMM/)
+		{	for my $fname (qw/TIT2 TPE1 TALB TDRC TYER TRCK TPOS TIT3 COMM/)
 			{	next unless exists $ID3v2->{frames}{$fname};
 				my $i=($fname eq 'COMM')? 2 : 0;
 				push @{ $tag[$TAGKEYS{$fname}] },$$_[$i] for @{ $ID3v2->{frames}{$fname} };
@@ -115,9 +115,8 @@ sub Read
 	 }
 	}
 	my $estimated;
-	if ($filetag->{info})
-	{	my $info=$filetag->{info};
-		$estimated=$info->{estimated};
+	if (my $info=$filetag->{info})
+	{	$estimated=$info->{estimated};
 		if ( exists $info->{seconds}
 		     && ( !$estimated || ($findlength && !$$aref[::SONG_LENGTH]) )
 		   )	#don't replace existing values for length and bitrate by estimated values (for mp3)
@@ -812,13 +811,11 @@ sub set
 	{	my $id3v2 = $tag->{ID3v2} || $tag->new_ID3v2;
 		for my $aref (@$modif)
 		{	my ($field,$add,@vals)=@$aref;
-			my $key=$TAGCODE{$field}[K_ID3V2];
+			my ($key,@extra)=split /-/,$TAGCODE{$field}[K_ID3V2],-1; #-1 to keep empty trailing fields #COMM-- => key="COMM" and @extra=("","")
 			@vals=@{$vals[0]} if ref $vals[0];
+			unshift @vals,@extra;
 			$id3v2->remove_all($key) unless $add;
 			next if $field eq ::SONG_GENRE && !@vals;
-			unshift @vals,$1 while $key=~s/-([^-]*)$//; # COMM-- => key is COMM and 2 '' unshifted in @vals
-			#if	($field==::SONG_GENRE)	{ next unless @vals; }
-			#elsif	($field==::SONG_COMMENT){unshift @vals,'','';}
 			$id3v2->insert($key,@vals);
 			if ($field eq ::SONG_DATE && $id3v2->{version}==4 && $vals[0]=~m/^\d{4}$/) { $id3v2->remove_all('TDRC'); $id3v2->insert('TDRC',$vals[0]); } #special-case: write a TDRC field for id3v2.4 in addition to TYER  #FIXME only support the yyyy format, could try to convert the date to yyyy-MM-ddTHH:mm:ss format
 		}
