@@ -1,18 +1,19 @@
-# Copyright (C) 2005-2007 Quentin Sculo <squentin@free.fr>
+# Copyright (C) 2005-2008 Quentin Sculo <squentin@free.fr>
 #
 # This file is part of Gmusicbrowser.
 # Gmusicbrowser is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 3, as
 # published by the Free Software Foundation
 
-#http://www.xiph.org/ogg/vorbis/doc/framing.html
-#http://www.xiph.org/ogg/vorbis/doc/Vorbis_I_spec.html
+#http://xiph.org/vorbis/doc/framing.html
+#http://xiph.org/vorbis/doc/v-comment.html
 
 package Tag::OGG;
 
 use strict;
 use warnings;
 use Encode qw(decode encode);
+use bytes;
 
 use constant
 { PACKET_INFO	 => 1,
