@@ -19,8 +19,14 @@ use Net::DBus;
 
 my $bus= Net::DBus->find;
 my $service = $bus->get_service('org.gnome.SettingsDaemon');
-my $object = $service->get_object('/org/gnome/SettingsDaemon');
-$object->connect_to_signal(MediaPlayerKeyPressed => \&callback);
+my $object = $service->get_object('/org/gnome/SettingsDaemon/MediaKeys');
+eval { $object->connect_to_signal(MediaPlayerKeyPressed => \&callback) };
+if ($@) {	my $error=$@;
+		# try with old path (for gnome version until ~2.20  <2.22)
+		$object=$service->get_object('/org/gnome/SettingsDaemon');
+		eval { $object->connect_to_signal(MediaPlayerKeyPressed => \&callback); };
+		die $error if $@; #die with the original error
+	}
 
 my %cmd=
 (	Previous	=> 'PrevSong',
