@@ -28,7 +28,7 @@ my %sites=	# id => [name,url,?post?,function]	if the function return 1 => lyrics
 	lyrc	=>	['lyrc','http://lyrc.com.ar/en/tema1en.php?artist=%a&songname=%s',undef,sub
 		{	local $_=$_[0];
 			return -1 if m#<a href=[^>]+add[^>]+>(?:[^<]*</?[b-z]\w*)*[^<]*Add a lyric.(?:[^<]*</?[b-z]\w*)*[^<]*</a>#i;
-			return 1 if m#<a href="\#"[^>]+badsong[^>]+>BADSONG</a>#i;
+			return 1 if s#<a href="\#"[^>]+badsong[^>]+>BADSONG</a>##i;
 			return 0;
 		}],
 	#leoslyrics =>	['leolyrics','http://api.leoslyrics.com/api_search.php?artist=%a&songtitle=%s'],
@@ -206,8 +206,22 @@ sub populate_popup_cb
 			if ($self->{widget_options}{HideToolbar}^=1)	{$self->{toolbar}->hide}
 			else						{$self->{toolbar}->show}
 		},$self);
-
 	$menu->append($_) for Gtk2::SeparatorMenuItem->new, $item0,$item1;
+
+	# add menu items for links
+	my ($x,$y)=$textview->window_to_buffer_coords('widget',$textview->get_pointer);
+	if (my $url=$self->url_at_coords($x,$y))
+	{	my $item2=Gtk2::MenuItem->new(_"Open link in Browser");
+		$item2->signal_connect(activate => sub	{ ::openurl($url); });
+		my $item3=Gtk2::MenuItem->new(_"Copy link address");
+		$item3->signal_connect(activate => sub
+		{	my $url=$_[1];
+			my $clipboard=$_[0]->get_clipboard(Gtk2::Gdk::Atom->new('CLIPBOARD',1));
+			$clipboard->set_text($url);
+		},$url);
+		$menu->prepend($_) for Gtk2::SeparatorMenuItem->new, $item3,$item2;
+	}
+
 	$menu->show_all;
 }
 
@@ -428,20 +442,20 @@ sub button_release_cb
 	my $self=::find_ancestor($textview,__PACKAGE__);
 	return ::FALSE unless $event->button == 1;
 	my ($x,$y)=$textview->window_to_buffer_coords('widget',$event->x, $event->y);
-	my $iter=$textview->get_iter_at_location($x,$y);
-	$self->follow_if_link($iter);
+	my $url=$self->url_at_coords($x,$y);
+	$self->load_url($url) if $url;
 	return ::FALSE;
 }
 
-sub follow_if_link
-{	my ($self,$iter)=@_;
+sub url_at_coords
+{	my ($self,$x,$y)=@_;
+	my $iter=$self->{textview}->get_iter_at_location($x,$y);
 	for my $tag ($iter->get_tags)
 	{	next unless $tag->{url};
 		if ($tag->{url}=~m/^#(\d+)?/) { $self->scrollto($1) if defined $1; last }
 		my $url= $self->full_url( $tag->{url} );
 		if ($url=~m#^http://www\.lyrc\.com\.ar/en/add/add\.php\?#) {::openurl( quotemeta($url) ); return}	#lyrc specific
-		$self->load_url($url);
-		last;
+		return $url;
 	}
 }
 
@@ -466,17 +480,15 @@ sub update_cursor_cb
 	my (undef,$wx,$wy,undef)=$textview->window->get_pointer;
 	my ($x,$y)=$textview->window_to_buffer_coords('widget',$wx,$wy);
 	my $iter=$textview->get_iter_at_location($x,$y);
-	my $cursor;
+	my $cursor='xterm';
 	for my $tag ($iter->get_tags)
 	{	next unless $tag->{url};
-		$cursor=1;
+		$cursor='hand2';
 		last;
 	}
-	if	($textview->{cursor} && !$cursor) { $textview->{cursor}=undef;	}
-	elsif	(!$textview->{cursor} && $cursor) { $textview->{cursor}=1;	}
-	else {return}
-	$cursor= Gtk2::Gdk::Cursor->new( $cursor? 'hand2' : 'xterm' );
-	$textview->get_window('text')->set_cursor($cursor);
+	return if ($textview->{cursor}||'') eq $cursor;
+	$textview->{cursor}=$cursor;
+	$textview->get_window('text')->set_cursor(Gtk2::Gdk::Cursor->new($cursor));
 }
 
 sub load_file
